@@ -39,6 +39,12 @@ class Widget(AstNode):
             self.group.tokens["name"].range,
         )
 
+    def get_reference(self, _idx: int) -> T.Optional[LocationLink]:
+        if obj := self.context[ScopeCtx].objects.get(self.name):
+            return LocationLink(self.range, obj.range, obj.ranges["id"])
+        else:
+            return None
+
     @validate("name")
     def obj_widget(self):
         object = self.context[ScopeCtx].objects.get(self.tokens["name"])
@@ -88,11 +94,30 @@ class ExtSizeGroupWidgets(AstNode):
     def unique_in_parent(self):
         self.validate_unique_in_parent("Duplicate widgets block")
 
+    @docs("widgets")
+    def ref_docs(self):
+        return get_docs_section("Syntax ExtSizeGroupWidgets")
+
 
 @completer(
     applies_in=[ObjectContent],
     applies_in_subclass=("Gtk", "SizeGroup"),
     matches=new_statement_patterns,
 )
-def size_group_completer(lsp, ast_node, match_variables):
-    yield Completion("widgets", CompletionItemKind.Snippet, snippet="widgets [$0]")
+def size_group_completer(_ctx: CompletionContext):
+    yield Completion(
+        "widgets",
+        CompletionItemKind.Snippet,
+        snippet="widgets [$0]",
+        sort_text=get_sort_key(CompletionPriority.OBJECT_MEMBER, "widgets"),
+    )
+
+
+@decompiler("widgets")
+def size_group_decompiler(ctx, gir: gir.GirContext):
+    ctx.print("widgets [")
+
+
+@decompiler("widget")
+def widget_decompiler(ctx, gir: gir.GirContext, name: str):
+    ctx.print(name + ",")
