@@ -24,10 +24,11 @@ import traceback
 import typing as T
 from difflib import SequenceMatcher
 
-from . import decompiler, formatter, parser, tokenizer, utils, xml_reader
+from . import decompiler, formatter, language, parser, tokenizer, utils, xml_reader
 from .ast_utils import AstNode
 from .completions import complete
 from .errors import CompileError, MultipleErrors
+from .linter import lint
 from .lsp_utils import *
 from .outputs.xml import XmlOutput
 from .tokenizer import Token
@@ -81,6 +82,8 @@ class OpenFile:
             self.diagnostics += warnings
             if errors is not None:
                 self.diagnostics += errors.errors
+            if self.ast is not None:
+                self.diagnostics += lint(self.ast)
         except MultipleErrors as e:
             self.diagnostics += e.errors
         except CompileError as e:
@@ -102,10 +105,10 @@ class OpenFile:
         ]
 
         # convert line, column numbers to deltas
-        for i, token_list in enumerate(token_lists[1:]):
-            token_list[0] -= token_lists[i][0]
-            if token_list[0] == 0:
-                token_list[1] -= token_lists[i][1]
+        for a, b in zip(token_lists[-2::-1], token_lists[:0:-1]):
+            b[0] -= a[0]
+            if b[0] == 0:
+                b[1] -= a[1]
 
         # flatten the list
         return [x for y in token_lists for x in y]
@@ -118,6 +121,7 @@ class LanguageServer:
         self.client_capabilities = {}
         self.client_supports_completion_choice = False
         self._open_files: T.Dict[str, OpenFile] = {}
+        self._exited = False
 
     def run(self):
         # Read <doc> tags from gir files. During normal compilation these are
@@ -125,7 +129,7 @@ class LanguageServer:
         xml_reader.PARSE_GIR.add("doc")
 
         try:
-            while True:
+            while not self._exited:
                 line = ""
                 content_len = -1
                 while content_len == -1 or (line != "\n" and line != "\r\n"):
@@ -149,7 +153,7 @@ class LanguageServer:
 
     def _send(self, data):
         data["jsonrpc"] = "2.0"
-        line = json.dumps(data, separators=(",", ":")) + "\r\n"
+        line = json.dumps(data, separators=(",", ":"))
         printerr("output: " + line)
         sys.stdout.write(
             f"Content-Length: {len(line.encode())}\r\nContent-Type: application/vscode-jsonrpc; charset=utf-8\r\n\r\n{line}"
@@ -209,6 +213,7 @@ class LanguageServer:
                     },
                     "completionProvider": {},
                     "codeActionProvider": {},
+                    "inlayHintProvider": {},
                     "hoverProvider": True,
                     "documentSymbolProvider": True,
                     "definitionProvider": True,
@@ -221,6 +226,14 @@ class LanguageServer:
             },
         )
 
+    @command("shutdown")
+    def shutdown(self, id, params):
+        self._send_response(id, None)
+
+    @command("exit")
+    def exit(self, id, params):
+        self._exited = True
+
     @command("textDocument/didOpen")
     def didOpen(self, id, params):
         doc = params.get("textDocument")
@@ -314,6 +327,53 @@ class LanguageServer:
 
         self._send_response(id, lst)
 
+    @command("textDocument/inlayHint")
+    def inlay_hints(self, id, params):
+        open_file = self._open_files[params["textDocument"]["uri"]]
+
+        if open_file.ast is None:
+            self._send_response(id, [])
+            return
+
+        range_start = utils.pos_to_idx(
+            params["range"]["start"]["line"],
+            params["range"]["start"]["character"],
+            open_file.text,
+        )
+        range_end = utils.pos_to_idx(
+            params["range"]["end"]["line"],
+            params["range"]["end"]["character"],
+            open_file.text,
+        )
+
+        hints = []
+
+        def collect_hints(node: AstNode):
+            if node.range.end < range_start or node.range.start > range_end:
+                return
+
+            start_line = utils.idx_to_pos(node.range.start, node.range.original_text)[0]
+            end_line = utils.idx_to_pos(node.range.end, node.range.original_text)[0]
+
+            # Don't show hints for very short blocks
+            if end_line - start_line < 3:
+                return
+
+            if isinstance(node, (language.Object)):
+                line, col = utils.idx_to_pos(node.range.end, node.range.original_text)
+                hints.append(
+                    {
+                        "position": {"line": line, "character": col},
+                        "label": " " + node.signature,
+                    }
+                )
+
+            for child in node.children:
+                collect_hints(child)
+
+        collect_hints(open_file.ast)
+        self._send_response(id, hints)
+
     @command("textDocument/x-blueprint-compile")
     def compile(self, id, params):
         open_file = self._open_files[params["textDocument"]["uri"]]
@@ -324,8 +384,8 @@ class LanguageServer:
 
         xml = None
         try:
-            output = XmlOutput()
-            xml = output.emit(open_file.ast, indent=2, generated_notice=False)
+            output = XmlOutput(generated_notice=False)
+            xml = output.emit(open_file.ast)
         except:
             printerr(traceback.format_exc())
             self._send_error(id, ErrorCode.RequestFailed, "Could not compile document")
@@ -381,24 +441,31 @@ class LanguageServer:
             open_file.text,
         )
 
+        def action_to_json(action: CodeAction, diagnostic: CompileError):
+            assert diagnostic.range is not None
+            edits = [
+                {
+                    "range": (
+                        action.edit_range.to_json()
+                        if action.edit_range
+                        else diagnostic.range.to_json()
+                    ),
+                    "newText": action.replace_with,
+                }
+            ]
+
+            for edit in action.additional_edits or []:
+                edits.append(edit.to_json())
+
+            return edits
+
         actions = [
             {
                 "title": action.title,
                 "kind": "quickfix",
                 "diagnostics": [self._create_diagnostic(open_file.uri, diagnostic)],
                 "edit": {
-                    "changes": {
-                        open_file.uri: [
-                            {
-                                "range": (
-                                    action.edit_range.to_json()
-                                    if action.edit_range
-                                    else diagnostic.range.to_json()
-                                ),
-                                "newText": action.replace_with,
-                            }
-                        ]
-                    }
+                    "changes": {open_file.uri: action_to_json(action, diagnostic)}
                 },
             }
             for diagnostic in open_file.diagnostics
@@ -472,9 +539,15 @@ class LanguageServer:
             ),
         }
 
-        if isinstance(err, DeprecationWarning):
+        if err.id is not None:
+            result["code"] = err.id
+
+        if isinstance(err, DeprecatedWarning):
             result["tags"] = [DiagnosticTag.Deprecated]
 
+        if isinstance(err, UnusedWarning):
+            result["tags"] = [DiagnosticTag.Unnecessary]
+
         if len(err.references) > 0:
             result["relatedInformation"] = [
                 {
